/*
 * stdlib.h
 * Modified for use in 15-410 at CMU
 * Zachary Anderson(zra)
 */

/*
 * Copyright (c) 1994, 1998 University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Flux OSKit.  The OSKit is free software, also known
 * as "open source;" you can redistribute it and/or modify it under the terms
 * of the GNU General Public License (GPL), version 2, as published by the Free
 * Software Foundation (FSF).  To explore alternate licensing terms, contact
 * the University of Utah at csl-dist@cs.utah.edu or +1-801-585-3271.
 * 
 * The OSKit is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GPL for more details.  You should have
 * received a copy of the GPL along with the OSKit; see the file COPYING.  If
 * not, write to the FSF, 59 Temple Place #330, Boston, MA 02111-1307, USA.
 */


#ifndef _STDLIB_H_
#define _STDLIB_H_

#ifndef _SIZE_T
#define _SIZE_T
typedef unsigned int size_t;
#endif

#ifndef NULL
#define NULL 0
#endif

#define RAND_MAX 0x7fffffff

#define MB_CUR_MAX	1	/* We don't do locales, so it's always 1. */
#define MB_LEN_MAX	1

/*
 * These function prototypes are roughly in the same order
 * as their descriptions in the ISO/ANSI C standard, section 7.10.
 */

#define atoi(str) ((int)atol(str))
long atol(const char *__str);
long strtol(const char *__p, char **__out_p, int __base);
unsigned long strtoul(const char *__p, char **__out_p, int __base);
double   strtod (const char *, char **);
double   atof (const char *);

int rand(void);
void srand(unsigned __seed);

void *_malloc(size_t __size);
void *_calloc(size_t __nelt, size_t __eltsize);
void *_realloc(void *__buf, size_t __new_size);
void _free(void *__buf);

void *malloc(size_t size);
void *calloc(size_t nelt, size_t eltsize);
void *realloc(void *buf, size_t new_size);
void free(void *buf);

#endif /* _STDLIB_H_ */
