/** @file register.h
 *  @brief Thread context registers.
 *
 *  Defines the register set of a thread.
 *
 *  @bug No known bugs.
 *
 *  @author Lee Salzman (lsalzman)
 */

#ifndef __KERN_REGISTER_H__
#define __KERN_REGISTER_H__

typedef unsigned int register_t;

typedef struct
{
	register_t EAX;
	register_t ECX;
	register_t EDX;
	register_t EBX;
	register_t EBP;
	register_t ESI;
	register_t EDI;
	register_t EIP;
	register_t CS;
	register_t EFLAGS;
	register_t ESP;
	register_t SS;
} Registers;

/** @brief Jumps into a specified register set.
 *
 * Sets the processor registers to a specified register set.
 *
 * @param registers the register set to install
 *
 * @return Void
 */
extern void registers_jump (Registers * registers);

#endif /* __KERN_REGISTER_H__ */
 
