/**
 * @file loader.c
 * @brief User program data loading.
 *
 * Implementings reading of user program data from the exec2obj apps.
 *
 * @author Lee Salzman (lsalzman)
 * @author Hassaan Khan (hkhan)
 */
/*@{*/

/* --- Includes --- */
#include <string.h>
#include <stdio.h>
#include <malloc.h>
#include <exec2obj.h>
#include <loader.h>
#include <elf.h>

/* --- Local function prototypes --- */


/**
 * Copies data from a file into a buffer.
 *
 * @param filename   the name of the file to copy data from
 * @param offset     the location in the file to begin copying from
 * @param size       the number of bytes to be copied
 * @param buf        the buffer to copy the data into
 *
 * @return returns the number of bytes copied on succes; -1 on failure
 */
int getbytes( const char *filename, int offset, int size, char *buf )
{
  const char *buffer;
  int i;
 
  for(i=0; i< exec2obj_userapp_count; i++){    
    if(strcmp(filename,exec2obj_userapp_TOC[i].execname )==0){      
      buffer= exec2obj_userapp_TOC[i].execbytes;

      memcpy(buf, buffer+offset, size);
      return size;
    }
  }
 
  return -1;
}

/*@}*/
