/** @file thread_fork.S
 *  @brief Implements thread_fork stub.
 *
 * Implements thread_fork as taking a code entry point
 * and a new stack to associate with the newly forked thread.
 * thread_fork does not return for the newly created thread.
 *
 * @author Lee Salzman (lsalzman)
 */

#include <syscall_int.h>
	
	.global thread_fork
thread_fork:	
  pushl %ebp                    #push base pointer on stack
  movl %esp, %ebp               #move stack pointer to base pointer
  pushl %ebx                    #save ebx register
  pushl %edi			#save edi register

				#store the paramester in registers so there is no worry about them being
				# overwritten before they are used
  movl 8(%ebp), %ebx            #the new thread entry point
  movl 12(%ebp), %edi		#the new thread stack

  int $THREAD_FORK_INT          #invoke the thread_fork trap

  testl %eax, %eax		#if the result is not 0, just return the result
  jnz .epilogue                 # otherwise set up the new entry point

  movl %edi, %esp		#establish the new stack pointer
  call *%ebx			#call the entry point

  pushl $-1
  call exit			#if the entry point returns, call exit(-1)

.busy_loop:
  jmp .busy_loop                #if that fails, then just do nothing
 
.epilogue:
  popl %edi			#restore edi register
  popl %ebx                     #restore ebx register
  movl %ebp, %esp               #move base pointer to stack pointer
  popl %ebp                     #pop off base pointer
  ret
.end
